#include "bloom_filter.h"

uint64_t calc_hash(const char* str, uint64_t modulus, uint64_t seed) {
    uint64_t hash = seed;
    for (int i = 0; str[i] != '\0'; ++i) {
        hash = (hash * 31 + str[i]) % modulus;
    }
    return hash;
}

void bloom_init(struct BloomFilter* bloom_filter, uint64_t set_size, hash_fn_t hash_fn, uint64_t hash_fn_count) {
    bloom_filter->set_size = set_size;
    bloom_filter->hash_fn = hash_fn;
    bloom_filter->hash_fn_count = hash_fn_count;
    bloom_filter->set = calloc((set_size >> 6) + (set_size % 64 != 0), sizeof(uint64_t));
}

void bloom_destroy(struct BloomFilter* bloom_filter) {
    free(bloom_filter->set);
    bloom_filter->set = NULL;
}

void bloom_insert(struct BloomFilter* bloom_filter, Key key) {
    for (uint64_t i = 0; i < bloom_filter->hash_fn_count; ++i) {
        uint64_t hash = bloom_filter->hash_fn(key, bloom_filter->set_size, i);
        uint64_t index = hash >> 6;
        uint64_t bit = hash & 63;
        bloom_filter->set[index] |= (1ULL << bit);
    }
}

bool bloom_check(struct BloomFilter* bloom_filter, Key key) {
    for (uint64_t i = 0; i < bloom_filter->hash_fn_count; ++i) {
        uint64_t hash = bloom_filter->hash_fn(key, bloom_filter->set_size, i);
        uint64_t index = hash >> 6;
        uint64_t bit = hash & 63;
        if (!(bloom_filter->set[index] & (1ULL << bit))) {
            return false;
        }
    }
    return true;
}
